@@ -30,14 +30,14 @@
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
-#
 
+from typing import Any, Dict, List, Optional, Set, Tuple  # noqa: F401
 
-from collections import defaultdict
-from univention.ldap_cache.cache import get_cache
+from univention.ldap_cache.cache import Cache, get_cache  # noqa: F401
 
 
 def _extract_id_from_dn(dn):
+	# type: (str) -> str
 	"""We know that this is wrong in general. But to speed up things
 	we do not use explode_dn from ldap.
 	We use the knowledge about users/user, groups/group, computers/computer objects:
@@ -51,13 +51,14 @@ def _extract_id_from_dn(dn):
 
 
 def groups_for_user(user_dn, consider_nested_groups=True, cache=None):
+	# type: (str, bool, Optional[Dict[str, Set[str]]]) -> List[str]
 	user_dn = user_dn.lower()
 	if cache is None:
 		_cache = get_cache()
-		cache = _cache.get_sub_cache('uniqueMembers').load()
-		cache = dict((key, set(val.lower() for val in values)) for key, values in cache.items())
+		subcache = _cache.get_sub_cache('uniqueMembers').load()
+		cache = {key: {val.lower() for val in values} for key, values in subcache.items()}
 	search_for_dns = [user_dn]
-	found = set()
+	found = set()  # type: Set[str]
 	while search_for_dns:
 		search_for = search_for_dns.pop().lower()
 		for member, dns in cache.items():
@@ -71,16 +72,17 @@ def groups_for_user(user_dn, consider_nested_groups=True, cache=None):
 
 
 def users_in_group(group_dn, consider_nested_groups=True, readers=(None, None), group_cache={}):
+	# type: (str, bool, Tuple[Optional[Any], Optional[Any]], Dict[str, List[str]]) -> List[str]
 	group_dn = group_dn.lower()
 	cache = get_cache()
 	member_uid_cache, unique_member_cache = [cache.get_sub_cache(name) for name in ['memberUids', 'uniqueMembers']]
 	with member_uid_cache.reading(readers[0]) as member_uid_reader, unique_member_cache.reading(readers[1]) as unique_member_reader:
-		ret = set()
+		ret = set()  # type: Set[str]
 		members = unique_member_cache.get(group_dn, unique_member_reader)
 		if not members:
 			return []
 		uids = member_uid_cache.get(group_dn, member_uid_reader) or []
-		uids = set([uid.lower() for uid in uids])
+		uids = {uid.lower() for uid in uids}
 		for member in members:
 			rdn = _extract_id_from_dn(member).lower()
 			if rdn in uids:
@@ -89,16 +91,17 @@ def users_in_group(group_dn, consider_nested_groups=True, readers=(None, None),
 				continue
 			else:
 				if consider_nested_groups:
-						if member in group_cache:
-								ret.update(group_cache[member])
-						else:
-								members = users_in_group(member, consider_nested_groups, readers=(member_uid_reader, unique_member_reader), group_cache=group_cache)
-								group_cache[member] = members
-								ret.update(members)
+					if member in group_cache:
+						ret.update(group_cache[member])
+					else:
+						members = users_in_group(member, consider_nested_groups, readers=(member_uid_reader, unique_member_reader), group_cache=group_cache)
+						group_cache[member] = members
+						ret.update(members)
 		return sorted(ret)
 
 
 def users_groups():
+	# type: () -> Dict[str, List[str]]
 	"""
 	Find all user-group relationship, including implicit ones:
 	if Group1 have Group2 as a subgroup, all users from Group2
@@ -108,13 +111,13 @@ def users_groups():
 	cache = get_cache()
 	member_uid_cache, unique_member_cache = [cache.get_sub_cache(name) for name in ['memberUids', 'uniqueMembers']]
 
-	group_users = {}
-	_group_cache = {}
+	group_users = {}  # type: Dict[str, List[str]]
+	_group_cache = {}  # type: Dict[str, List[str]]
 	with member_uid_cache.reading() as member_uid_reader, unique_member_cache.reading() as unique_member_reader:
 		for group in unique_member_cache.keys():
 			group_users[group] = users_in_group(group, readers=(member_uid_reader, unique_member_reader), group_cache=_group_cache)
 
-	res = {}
+	res = {}  # type: Dict[str, Set[str]]
 	for group, members in group_users.items():
 		for member in members:
 			groups = res.setdefault(member, set())
