@@ -30,9 +30,9 @@
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
-#
 
 from logging import getLogger
+from typing import Any, Mapping, Optional, Sequence  # noqa: F401
 
 from univention.ldap_cache.cache import get_cache
 from univention.listener.handler import ListenerModuleHandler
@@ -40,6 +40,7 @@ from univention.listener.handler import ListenerModuleHandler
 
 class LdapCacheHandler(ListenerModuleHandler):
 	def __init__(self, *args, **kwargs):
+		# type: (*Any, **Any) -> None
 		self._counter = 0
 		super(LdapCacheHandler, self).__init__(*args, **kwargs)
 		cache_logger = getLogger('univention.ldap_cache')
@@ -48,31 +49,35 @@ class LdapCacheHandler(ListenerModuleHandler):
 			cache_logger.addHandler(handler)
 
 	def _cleanup_cache_if_needed(self):
+		# type: () -> None
 		self._counter += 1
 		if self._counter % 1000 == 0:
 			for name, db in get_cache():
 				db.cleanup()
 
 	def create(self, dn, new):
-		for shard in get_cache().get_shards_for_query(self._get_configuration().get_ldap_filter()):
+		# type: (str, Mapping[str, Sequence[bytes]]) -> None
+		for shard in get_cache().get_shards_for_query(self.config.get_ldap_filter()):
 			shard.add_object((dn, new))
 		self._cleanup_cache_if_needed()
 
 	def modify(self, dn, old, new, old_dn):
-		for shard in get_cache().get_shards_for_query(self._get_configuration().get_ldap_filter()):
+		# type: (str, Mapping[str, Sequence[bytes]], Mapping[str, Sequence[bytes]], Optional[str]) -> None
+		for shard in get_cache().get_shards_for_query(self.config.get_ldap_filter()):
 			shard.rm_object((old_dn or dn, old))
 			shard.add_object((dn, new))
 		self._cleanup_cache_if_needed()
 
 	def remove(self, dn, old):
-		for shard in get_cache().get_shards_for_query(self._get_configuration().get_ldap_filter()):
+		# type: (str, Mapping[str, Sequence[bytes]]) -> None
+		for shard in get_cache().get_shards_for_query(self.config.get_ldap_filter()):
 			shard.rm_object((dn, old))
 		self._cleanup_cache_if_needed()
 
 	def post_run(self):
+		# type: () -> None
 		self._counter = -1
 		self._cleanup_cache_if_needed()
 
 	class Configuration(ListenerModuleHandler.Configuration):
-		def get_priority(self):
-			return 2.0
+		priority = 2.0
