@@ -30,17 +30,18 @@
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
-#
-
 
 import logging
+from typing import Any  # noqa: F401
 
 
 def log(*msgs):
+	# type: (*Any) -> None
 	logger = logging.getLogger('univention.ldap_cache')
 	logger.info(*msgs)
 
 
 def debug(*msgs):
+	# type: (*Any) -> None
 	logger = logging.getLogger('univention.ldap_cache')
 	logger.debug(*msgs)
